

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators && is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Pivot Points.
	/// </summary>
	[Description("Swing Pivots according to EasyLanguage Function named Pivot")]
	public class Pivot_EasyLanguage : Indicator
	{
		#region Local Variables
			private double CandidatePrice=( 0 );
			private int LengthCntr=( 0 );
			private int StrengthCntr=( 0 ); 
			private int InstanceCntr=( 0 );
			private bool PivotTest=false;
			private bool InstanceTest=false;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.Blue,2), PlotStyle.Dot, "PivotPrice"));
			Add(new Plot(Color.Transparent,	"PivotBarRelative"));
			AutoScale = false;
			Overlay	= true;
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if(CurrentBar<pLeftStrength+pRightStrength+pLength) return;

			InstanceCntr = 0;
			InstanceTest = false;
			LengthCntr = pRightStrength; 
			while (LengthCntr < pLength && !InstanceTest) {
				// GET NEXT MOST RECENT CandidatePrice & SET STARTING ASSUMPTION
				// THAT IT IS A Pivot:
				CandidatePrice = (HiLo==1?High[LengthCntr]:Low[LengthCntr]) ;
				PivotTest = true;
				// COMPARE CandidatePrice TO LeftStrength BARS ON LEFT:
				StrengthCntr = LengthCntr + 1;
				while (PivotTest && StrengthCntr - LengthCntr <= pLeftStrength ) {
					if (( HiLo == 1 && CandidatePrice < High[StrengthCntr] ) || ( HiLo == -1 && CandidatePrice > Low[StrengthCntr] ))
						PivotTest = false;
					else
						StrengthCntr = StrengthCntr + 1;
				}
				// IF CandidatePrice SURVIVED LEFT COMPARISON, NOW COMPARE TO
				// RightStrength BARS ON RIGHT: }
				StrengthCntr = LengthCntr - 1;
				while (PivotTest && LengthCntr - StrengthCntr <= pRightStrength) {
					if ((pHiLo == 1 && CandidatePrice <= High[StrengthCntr] ) || (pHiLo == -1 && CandidatePrice >= Low[StrengthCntr] ))
						PivotTest = false;
					else
						StrengthCntr = StrengthCntr - 1;
				}
				// IF CandidatePrice SURVIVED OVERALL AS Pivot, INCREMENT InstanceCntr: 
				if (PivotTest) InstanceCntr = InstanceCntr + 1;
				// IF InstanceCntr HAS REACHED REQUIRED COUNT, THEN THIS IS THE DESIRED Pivot,
				// ELSE INCREMENT LengthCntr TO PREPARE TO GET && TEST NEXT CANDIDATE PRICE:
				if (InstanceCntr == Instance) InstanceTest = true;
				else LengthCntr = LengthCntr + 1;
			}
			if (InstanceTest) {
				PivotPrice.Set(CandidatePrice);
				PivotBarRelative.Set(LengthCntr);
			} else {
				PivotPrice.Set(-1);
				PivotBarRelative.Set(-1);
			}
		}
		#region Properties

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PivotPrice
		{
			get { return Values[0]; }
		}
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PivotBarRelative
		{
			get { return Values[1]; }
		}

		private int pLength=4; 
		[Description("Sest the number of bars to consider for the pivot")]
		[Category("Parameters")]
		public int Length
		{
			get { return pLength; }
			set { pLength = value; }
		}

		private int pLeftStrength=4;
		[Description("Bars on left whose price is at or away from current price")]
		[Category("Parameters")]
		public int LeftStrength
		{
			get { return pLeftStrength; }
			set { pLeftStrength = Math.Max(1, value); }
		}

		private int pRightStrength=4;
		[Description("Bars on left whose price is away from current price")]
		[Category("Parameters")]
		public int RightStrength
		{
			get { return pRightStrength; }
			set { pRightStrength = Math.Max(1, value); }
		}

		private int pInstance=1;
		[Description("1=most recent pivot, 2=prior pivot, etc")]
		[Category("Parameters")]
		public int Instance
		{
			get { return pInstance; }
			set { pInstance = Math.Max(1, value); }
		}

		private int pHiLo=1; 
		[Description("Pass in 1 for PivotHigh(Bar), -1 for PivotLow(Bar)")]
		[Category("Parameters")]
		public int HiLo
		{
			get { return pHiLo; }
			set { if(value!=1) pHiLo=-1; else pHiLo = value; }
		}

		#endregion
	}
}
#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private Pivot_EasyLanguage[] cachePivot_EasyLanguage = null;

        private static Pivot_EasyLanguage checkPivot_EasyLanguage = new Pivot_EasyLanguage();

        /// <summary>
        /// Swing Pivots according to EasyLanguage Function named Pivot
        /// </summary>
        /// <returns></returns>
        public Pivot_EasyLanguage Pivot_EasyLanguage(int hiLo, int instance, int leftStrength, int length, int rightStrength)
        {
            return Pivot_EasyLanguage(Input, hiLo, instance, leftStrength, length, rightStrength);
        }

        /// <summary>
        /// Swing Pivots according to EasyLanguage Function named Pivot
        /// </summary>
        /// <returns></returns>
        public Pivot_EasyLanguage Pivot_EasyLanguage(Data.IDataSeries input, int hiLo, int instance, int leftStrength, int length, int rightStrength)
        {
            checkPivot_EasyLanguage.HiLo = hiLo;
            hiLo = checkPivot_EasyLanguage.HiLo;
            checkPivot_EasyLanguage.Instance = instance;
            instance = checkPivot_EasyLanguage.Instance;
            checkPivot_EasyLanguage.LeftStrength = leftStrength;
            leftStrength = checkPivot_EasyLanguage.LeftStrength;
            checkPivot_EasyLanguage.Length = length;
            length = checkPivot_EasyLanguage.Length;
            checkPivot_EasyLanguage.RightStrength = rightStrength;
            rightStrength = checkPivot_EasyLanguage.RightStrength;

            if (cachePivot_EasyLanguage != null)
                for (int idx = 0; idx < cachePivot_EasyLanguage.Length; idx++)
                    if (cachePivot_EasyLanguage[idx].HiLo == hiLo && cachePivot_EasyLanguage[idx].Instance == instance && cachePivot_EasyLanguage[idx].LeftStrength == leftStrength && cachePivot_EasyLanguage[idx].Length == length && cachePivot_EasyLanguage[idx].RightStrength == rightStrength && cachePivot_EasyLanguage[idx].EqualsInput(input))
                        return cachePivot_EasyLanguage[idx];

            Pivot_EasyLanguage indicator = new Pivot_EasyLanguage();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.HiLo = hiLo;
            indicator.Instance = instance;
            indicator.LeftStrength = leftStrength;
            indicator.Length = length;
            indicator.RightStrength = rightStrength;
            indicator.SetUp();

            Pivot_EasyLanguage[] tmp = new Pivot_EasyLanguage[cachePivot_EasyLanguage == null ? 1 : cachePivot_EasyLanguage.Length + 1];
            if (cachePivot_EasyLanguage != null)
                cachePivot_EasyLanguage.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cachePivot_EasyLanguage = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Swing Pivots according to EasyLanguage Function named Pivot
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Pivot_EasyLanguage Pivot_EasyLanguage(int hiLo, int instance, int leftStrength, int length, int rightStrength)
        {
            return _indicator.Pivot_EasyLanguage(Input, hiLo, instance, leftStrength, length, rightStrength);
        }

        /// <summary>
        /// Swing Pivots according to EasyLanguage Function named Pivot
        /// </summary>
        /// <returns></returns>
        public Indicator.Pivot_EasyLanguage Pivot_EasyLanguage(Data.IDataSeries input, int hiLo, int instance, int leftStrength, int length, int rightStrength)
        {
            return _indicator.Pivot_EasyLanguage(input, hiLo, instance, leftStrength, length, rightStrength);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Swing Pivots according to EasyLanguage Function named Pivot
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Pivot_EasyLanguage Pivot_EasyLanguage(int hiLo, int instance, int leftStrength, int length, int rightStrength)
        {
            return _indicator.Pivot_EasyLanguage(Input, hiLo, instance, leftStrength, length, rightStrength);
        }

        /// <summary>
        /// Swing Pivots according to EasyLanguage Function named Pivot
        /// </summary>
        /// <returns></returns>
        public Indicator.Pivot_EasyLanguage Pivot_EasyLanguage(Data.IDataSeries input, int hiLo, int instance, int leftStrength, int length, int rightStrength)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.Pivot_EasyLanguage(input, hiLo, instance, leftStrength, length, rightStrength);
        }

    }
}
#endregion
